package cn.estsh.i3plus.core.apiservice.controller.report;

import cn.estsh.i3plus.core.api.iservice.base.IReportService;
import cn.estsh.i3plus.icloud.core.sdk.ICoreReportPojoCloud;
import cn.estsh.i3plus.platform.common.tool.ClassTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseResultBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.model.common.ClassModel;
import cn.estsh.i3plus.pojo.model.report.BrDataSourceModel;
import cn.estsh.i3plus.pojo.model.report.TemplateModel;
import cn.estsh.impp.framework.boot.util.ResultBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @Description :
 * @Reference :
 * @Author : wei.peng
 * @CreateDate : 19-12-5 下午5:25
 * @Modify:
 **/
@RestController
@Api(tags = "Cloud 报表服务")
@RequestMapping(CommonConstWords.BASE_URL_CORE + "/cloud/report-pojo")
public class ReportPojoCloudController {

    private static final Logger LOGGER = LoggerFactory.getLogger(ReportPojoCloudController.class);

    @Value("${impp.app.pojo-packages}.bean")
    private String pojoPackagePath;

    @Autowired
    private IReportService reportService;

    @PostMapping(value = "/find-data-source")
    @ApiOperation(value = "数据源列表", notes = "获取服务的数据源列表")
    public BaseResultBean<BrDataSourceModel> findDataSource() {
        return ResultBean.success("操作成功")
                .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                .setResultList(Arrays.asList(BrDataSourceModel.DEFAULT_DATA_SOURCE));
    }

    @PostMapping(value = "/find-pojo-detail")
    @ApiOperation(value = "获取对象列表", notes = "获取对象列表包含对象属性信息")
    public BaseResultBean<ClassModel> findCloudPojoDetail(@RequestBody BrDataSourceModel model) {
        LOGGER.info("【类路径：{}】", pojoPackagePath);
        List<String> clzNameList = ClassTool.getClassName(pojoPackagePath, true);
        LOGGER.info("【类信息：{}】", clzNameList);
        List<ClassModel> clzList = new ArrayList<>(clzNameList.size());
        for (String clzName : clzNameList) {
            ClassModel classModel = ClassTool.getClassModel(clzName, CommonEnumUtil.SOFT_TYPE.REPORT);
            if (classModel != null) {
                clzList.add(classModel);
            }
        }
        return ResultBean.success("获取类：" + clzList.size()).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultList(clzList);
    }

    /**
     * 根据核心包路径列出所有类
     *
     * @param model 模板模型
     * @return 处理结果
     */
    @PostMapping(value = "/hql/run")
    @ApiOperation(value = "查询管理后台所有类", notes = "根据核心包路径列出所有类")
    public BaseResultBean runHql(@RequestBody TemplateModel model) {
        LOGGER.info("【执行 HQL】TemplateModel：{}", model);
        try {
            List result = reportService.selectTemplateModel(model);
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(result);
        } catch (Exception e) {
            return ResultBean.fail("操作失败").setMsg(e.getMessage()).setErrorMsg(e.getLocalizedMessage());
        }
    }


    /**
     * 根据核心包路径列出所有类
     *
     * @param model 模板模型
     * @return 处理结果
     */
    @PostMapping(value = "/hql/run-count")
    @ApiOperation(value = "查询管理后台所有类", notes = "根据核心包路径列出所有类")
    public BaseResultBean runHqlCount(@RequestBody TemplateModel model) {
        LOGGER.info("【执行 HQL】TemplateModel：{}", model);
        try {
            Long count = reportService.countTemplateModel(model);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(count);
        } catch (Exception e) {
            return ResultBean.fail("操作失败").setMsg(e.getMessage()).setErrorMsg(e.getLocalizedMessage());
        }
    }
}
