package cn.estsh.i3plus.core.apiservice.schedulejob;

import cn.estsh.i3plus.core.api.iservice.busi.ISysLogExceptionService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysLogOperateService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysLogSystemService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysLogTaskTimeService;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.impp.framework.base.schedule.BaseImppScheduleJob;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import com.alibaba.fastjson.JSONObject;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import java.util.Calendar;
import java.util.Date;

/**
 * @Description :
 * @Reference : 日志清除
 * @Author : yunhao
 * @CreateDate : 2019-07-04 19:13
 * @Modify:
 **/
@DisallowConcurrentExecution
public class CoreLogClearJob extends BaseImppScheduleJob {

    public static final Logger LOGGER = LoggerFactory.getLogger(CoreLogClearJob.class);
    private static final long serialVersionUID = 2706787898778108791L;

    public CoreLogClearJob() {
        super(CoreLogClearJob.class, "日志清除");
    }

    @Value("${impp.log.retention-time:-1}")
    private Integer retentionTime;

    @Autowired
    private ISysLogSystemService sysLogSystemService;

    @Autowired
    private ISysLogExceptionService sysLogExceptionService;

    @Autowired
    private ISysLogOperateService sysLogOperateService;

    @Autowired
    private ISysLogTaskTimeService sysLogTaskTimeService;

    @Override
    public void executeImppJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        // 判断日志保留时间 -1 永久
        if (getJobParam() == null) {
            return;
        }
        JSONObject clearParam = JSONObject.parseObject(getJobParam());

        // 计算保留日期
        String dateTime =  TimeTool.timeCalc(new Date(), Calendar.DAY_OF_MONTH, -retentionTime,"yyyy-MM-dd HH:mm:ss");
        long count;
        // 系统日志
        if (clearParam.containsKey("logSystem") && clearParam.getInteger("logSystem") != -1) {
            dateTime =  TimeTool.timeCalc(new Date(), Calendar.DAY_OF_MONTH, -clearParam.getInteger("logSystem"),"yyyy-MM-dd HH:mm:ss");
            count = sysLogSystemService.deleteSysLogSystemByDateTimeAfter(dateTime);
            LOGGER.info("【日志清理】 清理系统日志 {} 条", count);
        }

        // 异常日志
        if (clearParam.containsKey("logException") && clearParam.getInteger("logException") != -1) {
            dateTime =  TimeTool.timeCalc(new Date(), Calendar.DAY_OF_MONTH, -clearParam.getInteger("logException"),"yyyy-MM-dd HH:mm:ss");
            count = sysLogExceptionService.deleteSysLogExceptionByDateTimeAfter(dateTime);
            LOGGER.info("【日志清理】 清理异常日志 {} 条", count);
        }

        // 操作日志
        if (clearParam.containsKey("logOperate") && clearParam.getInteger("logOperate") != -1) {
            dateTime =  TimeTool.timeCalc(new Date(), Calendar.DAY_OF_MONTH, -clearParam.getInteger("logException"),"yyyy-MM-dd HH:mm:ss");
            count = sysLogOperateService.deleteSysLogOperateByDateTimeAfter(dateTime);
            LOGGER.info("【日志清理】 清理操作日志 {} 条", count);
        }

        // 任务日志
        if (clearParam.containsKey("logTask") && clearParam.getInteger("logTask") != -1) {
            dateTime =  TimeTool.timeCalc(new Date(), Calendar.DAY_OF_MONTH, -clearParam.getInteger("logException"),"yyyy-MM-dd HH:mm:ss");
            count = sysLogTaskTimeService.deleteSysLogTaskTimeByDateTimeAfter(dateTime);
            LOGGER.info("【日志清理】 清理定时任务日志 {} 条", count);
        }
    }

    public static void main(String[] args) {
        JSONObject clearParam = new JSONObject();
        clearParam.put("logSystem",30);
        clearParam.put("logException",30);
        clearParam.put("logOperate",30);
        clearParam.put("logTask",30);
        System.out.println(clearParam.toJSONString());


        clearParam = JSONObject.parseObject("{\"logOperate\":10,\"logException\":10,\"logSystem\":10,\"logTask\":10}");

        // 计算保留日期
        String dateTime = TimeTool.pareDateToString("yyyy-MM-dd HH:mm:ss",
                TimeTool.timeCalc(new Date(), Calendar.DAY_OF_MONTH, 1)
        );
    }
}
